package lab.game;

import java.util.Random;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;
import javafx.scene.transform.Affine;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Transform;

public class Rock  extends WorldEntity {

	private static final Random RANDOM = new Random();
	private MyDimension size;
	private double angle = 0;

	public Rock(Scene scene) {
		this(scene, new MyPoint(350, 200), new MyDimension(60, 10));
	}

	public Rock(Scene scene, MyPoint position, MyDimension size) {
		super(scene, position, 50);
		this.size = size;
	}

	@Override
	public void drawInternal(GraphicsContext gc) {
		gc.setFill(Color.GRAY);
		gc.setStroke(Color.GREEN);
		MyPoint center = position.add(size.getWidth() / 2, size.getHeight() / 2);
		Rotate rotateMatrix = Transform.rotate(angle, center.getX(), center.getY());
		gc.setTransform(new Affine(rotateMatrix));
		gc.fillRect(position.getX(), position.getY(), size.getWidth(), size.getHeight());
		gc.strokeRect(position.getX(), position.getY(), size.getWidth(), size.getHeight());
	}

	public void simulate(double deltaTime) {
		angle += 0.5;
	}

	public void changePosition() {
		position = new MyPoint(
				RANDOM.nextDouble(scene.getSize().getWidth()*0.3, scene.getSize().getWidth()*0.9),
				RANDOM.nextDouble(scene.getSize().getHeight()*0.5, scene.getSize().getHeight()*0.95));
	}

}
