package lab.game;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import javafx.scene.canvas.GraphicsContext;
import lab.Setup;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class Scene {

	private Object entitiesLock = new Object();

	private boolean viewMode;

	private MyDimension size;
	private List<DrawableSimulable> sceneEntitites;
	private List<DrawableSimulable> entititesToAdd = new ArrayList<>();
	private List<DrawableSimulable> entititesToRemove = new ArrayList<>();
	private List<LochNessDeadLog> lochNessDeadLogs = new LinkedList<>();

	private Boat boat;
	private Rock rock;
	private Background background;

	public Scene(double width, double height, int numberOfMonsters, boolean viewMode) {
		size = new MyDimension(width, height);
		this.viewMode = viewMode;
		sceneEntitites = new ArrayList<>();
		background = new Background(this);
		rock = new Rock(this, new MyPoint(300, 300), new MyDimension(30, 50));
		boat = new Boat(this, new MyPoint(20, 200));
		sceneEntitites.add(background);
		sceneEntitites.add(rock);
		sceneEntitites.add(boat);
		for (int i = 0; i < numberOfMonsters * Setup.getInstance().getLochnessMultiplier(); i++) {
			LochNess lochNess = new LochNess(this);
			lochNess.addDeadListener(
					ln -> lochNessDeadLogs.add(new LochNessDeadLog(LocalDateTime.now(), ln.getPosition())));
			sceneEntitites.add(lochNess);
		}
		initServerOrViewMode();
	}

	public void initServerOrViewMode() {
		if (!viewMode) {
			startServer();
		} else {
			connectToServer();
		}
	}

	public void startServer() {
		//TODO:
	}

	public void connectToServer() {
		//TODO:
	}

	record LochNessDeadLog(LocalDateTime time, MyPoint position) {
	}

	public void printDeadLog() {
		for (LochNessDeadLog lochNessDeadLog : lochNessDeadLogs) {
			log.info(lochNessDeadLog);
		}
	}

	public MyDimension getSize() {
		return size;
	}

	public void draw(GraphicsContext gc) {
		synchronized (entitiesLock) {
			for (DrawableSimulable entity : sceneEntitites) {
				entity.draw(gc);
			}
		}
	}

	public void simulate(double deltaTime) {
		if (viewMode) {
			return;
		}
		synchronized (entitiesLock) {
			for (DrawableSimulable drawableSimulable : sceneEntitites) {
				drawableSimulable.simulate(deltaTime);
			}
			detectColision();
			sceneEntitites.removeAll(entititesToRemove);
			sceneEntitites.addAll(entititesToAdd);
			entititesToAdd.clear();
			entititesToRemove.clear();
			sceneEntitites.sort(Comparator.comparing(DrawableSimulable::getZIndex));
		}
	}

	private void detectColision() {
		for (int i = 0; i < sceneEntitites.size(); i++) {
			if (sceneEntitites.get(i) instanceof Collisionable c1) {
				for (int j = i + 1; j < sceneEntitites.size(); j++) {
					if (sceneEntitites.get(j) instanceof Collisionable c2 && c1.intersect(c2.getBoundingBox())) {
						c1.hitBy(c2);
						c2.hitBy(c1);
					}
				}
			}
		}
	}

	public Boat getBoat() {
		return boat;
	}

	public Rock getRock() {
		return rock;
	}

	public Background getBackground() {
		return background;
	}

	public void add(DrawableSimulable entity) {
		if (viewMode) {
			return;
		}
		entititesToAdd.add(entity);
	}

	public void remove(DrawableSimulable entity) {
		if (viewMode) {
			return;
		}
		entititesToRemove.add(entity);
	}

}
