package lab.gui;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.canvas.Canvas;
import javafx.scene.control.Label;
import javafx.scene.control.Slider;
import lab.game.DrawingThread;
import lab.game.LochNess;
import lab.game.Scene;

public class GameController {

	private static Logger log = LogManager.getLogger(GameController.class);
	
	private Scene scene;
	private boolean viewMode; 

    @FXML
	private Slider boatPosition;

	private DrawingThread timer;

	@FXML
	private Canvas canvas;

	@FXML
	private Slider speed;

	@FXML
	private Label playerName;

	private int deadLochNessCounter = 0;
	
	@FXML
	void changePosition(ActionEvent event) {
		LochNess lochNess = new LochNess(scene);
		lochNess.addDeadListener(ln -> log.info("LochNess dead!"));
		lochNess.addDeadListener(this::addDead);
		scene.add(lochNess);

	}
	
	private void addDead(LochNess lochNess) {
		deadLochNessCounter++;
		updateDeadLabel();
	}

	private void updateDeadLabel() {
		playerName.setText(String.format("Number of dead lochnesses: %03d", deadLochNessCounter));
	}

	@FXML
	void initialize() {
		assert boatPosition != null : "fx:id=\"angle\" was not injected: check your FXML file 'gameWindow.fxml'.";
		assert canvas != null : "fx:id=\"canvas\" was not injected: check your FXML file 'gameWindow.fxml'.";
		assert speed != null : "fx:id=\"speed\" was not injected: check your FXML file 'gameWindow.fxml'.";
		boatPosition.valueProperty().addListener(new ChangeListener<>() {
			@Override
			public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number newValue) {
				scene.getBoat().setPosInPercentage(newValue.doubleValue());
			}
		});
		speed.valueProperty().addListener(new ChangeListener<>() {
			@Override
			public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number newValue) {
				scene.getBackground().setSpeed(newValue.doubleValue());
			}
		});
		log.info("Screeen initialized.");
	}

	public void setViewMode(boolean viewMode) {
		this.viewMode = viewMode;
	}

	public void startGame(String name, int numberOfMonsters) {
		playerName.setText(name);
		scene = new Scene(canvas.getWidth(), canvas.getHeight(), numberOfMonsters, viewMode);
		timer = new DrawingThread(canvas, scene);
		timer.start();

	}

	public void stop() {
		timer.stop();
		scene.printDeadLog();
	}

}
